<?php
/**
 * login.php - Survey login controller.
 *
 * @author $Author: suppakit $
 * @version $Id: login.php,v 1.14 2012/05/10 07:01:27 suppakit Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/

class Login extends CI_Controller {
	const LINK_SHOW_SURVEY = 'survey/survey/index/';
	const LINK_SURVEY_LOGIN = 'survey/login/index/';
	private $survey_parity = 0;
	private $survey_id = 0;
	private $survey_code = FALSE;
	private $error_message = FALSE;
	private $survey_expired = FALSE;
	private $survey_record = FALSE;
	private $user = FALSE;

	function __construct() {
		parent::__construct();
		osa_pureobject('survey_login_class', TRUE);
		$this->load->model('auth/survey_login_model');
		$this->load->model('survey/survey_model');
		$this->load->helper(array('form', 'url'));
      $this->load->library('form_validation');
      osa_load_lang('login');
      osa_load_lang('survey');
	}

   function index($survey_code=FALSE) {
   	//Add survey code to class variable
   	$this->survey_code = $survey_code;
   	if ( array_key_exists('surveyid', $_REQUEST) ) {
   		$this->survey_code = $_REQUEST['surveyid'];
   	}

   	if ( !($this->survey_record = $this->_is_valid_survey()) ) {
   		//Invalid survey due to incorrect survey code or expired survey
   		//This check is performed regardless of type of login for the survey
   		$this->_show_login_page();
   		return;
   	}
   	if ( $this->survey_record === TRUE ) {
   		redirect($this->_show_survey_link($this->survey_code));
      	exit;
   	}
   	//We can assume the survey code is valid after this point and all survey info and record are available

   	$open_survey_flag = $this->_is_open_survey();
   	if ( ($open_survey_flag || $this->survey_login_model->isAuthenticated()) && !$this->_require_security_code() ) {
   		//Survey is an open survey without code or user is already login so redirect to the actual survey page
   		if ( $open_survey_flag ) {
   			//Create an empty login object if no login object yet for open surveys
   			$this->survey_login_model->init_empty_login();
   		}
   		//Add survey info to login object so survey.php knows what to show
			$this->_add_survey2login();
     		redirect($this->_show_survey_link());
      	exit;
   	}
		//Have to process login or security code to continue
   	if ( $this->_require_security_code() ) {
			$this->_get_security_code();
			return;
		}
		else {
   		$this->_password_login();
   		return;
		}
	}

   function logout() {
      $this->survey_login_model->destroyLogin();
      redirect('survey/login/index');
      exit;
   	}

   private function _get_security_code() {
   	if ( array_key_exists('securitycode', $_REQUEST) ) {
   		$user_code = $_REQUEST['securitycode'];
   		if ( !empty($user_code) ) {
				if ( strtolower($user_code) == strtolower($this->survey_record->code) ) {
					$this->survey_login_model->init_empty_login();
					$this->_add_survey2login();
            	redirect($this->_show_survey_link());
      	exit;
   	}
				else {
					//Invalid code
					$this->error_message = lang('survey_invalid_security_code');
				}
   		}
   		else {
				//Missing code
				$this->error_message = lang('survey_missing_security_code');
   		}
   	}
		$this->_show_login_page(TRUE);
   }

   private function _password_login() {
      $this->form_validation->set_rules('username', 'Username',
         										 'trim|required|min_length[1]|strtolower');
      //Can't use a variable to set lenght of password, has to be hardcoded here
      $this->form_validation->set_rules('password', 'Password', 'required|min_length[1]');
      $this->form_validation->set_rules('surveyid', 'Survey Number', 'trim|required|min_length[1]');
		$form_result = $this->form_validation->run();
		$submitted_surveyid = set_value('surveyid');
      if ( $form_result == FALSE ) {
      	$username = set_value('username');
      	$password = set_value('password');
      	if ( (empty($username) || empty($password)) && array_key_exists('username', $_POST) ) {
      		//Only display error for form-submit/post
      		$this->error_message = lang('login_invalidlogin');
      	}
      	$this->_show_login_page();
      	return;
      }
      else {
      	//Got all the form elements and do the authentication
   	  	$username = set_value('username');
      	$password = set_value('password');
      	$username = osa_login_username_convert(osa_des_decrypt($username));
      	$password = osa_des_decrypt($password);
      	if ( $this->survey_login_model->authenticate($username, $password) ) {
      		$this->_add_survey2login();
            redirect($this->_show_survey_link());
            exit;
      	}
      	else {
				//$this->session->set_flashdata('AUTH_FAILED_USERNAME', $username);
				//redirect($this->_survey_login_link());
				$this->error_message = lang('login_invalidlogin');
				$this->_show_login_page();
            return;
        }
      }
   }

   private function _show_login_page($show_security_code=FALSE) {
   	//$this->survey_login_model->destroyLogin();
   	//Override the login page based on the admin configs
   	if ( !$this->config->item('cms_survey_uselogin') ) {
   		$show_security_code = TRUE;
   	}
  	 	if ( !$this->config->item('cms_survey_nologin') ) {
   		$show_security_code = FALSE;
   	}
   	//Override the login page again based on the actual survey
   	if ( is_object($this->survey_record) && property_exists($this->survey_record, 'islogin') ) {
			if ( $this->survey_record->islogin ) {
				$show_security_code = FALSE;
			}
			else {
				$show_security_code = TRUE;
			}
   	}

   	$data = array();
   	$data['site_title'] = $this->config->item('cms_survey_login_title');
   	$data['username'] = '';
   	if ( array_key_exists('username', $_REQUEST) ) {
   		// Now we use encrypted login so we must decrypt the _REQUEST
   		$decrypted_user = osa_des_decrypt($_REQUEST['username']);
   		$data['username'] = $decrypted_user;
   	}
      //_maintenance_message() can reset $this->error_message so make sure $data['msg_mode'] comes before $data['msg']
      $data['msg_mode'] = $this->_maintenance_message();
      $data['msg'] = $this->error_message;
      $data['surveyid'] = $this->survey_code;
      $data['form_link'] = $this->_survey_login_link();
      $data['show_security_code'] = $show_security_code;
      // Override the logo image if required.
		osa_login_logo_override();
      $this->load->view('survey/login_m_view', $data);
      return;
   }

   private function _maintenance_message() {
   	$msg = FALSE;
      if ( $this->config->item('cms_manage_mode') ){
      	 $msg = lang('login_managemode') ;
      }
      if (  $this->config->item('cms_sitestop') ) {
      	$msg = lang('gen_maintenance_msg');
   }

      $sess_managemode = $this->session->flashdata('AUTH_FAILED_MANAGE_MODE');
      if ( $sess_managemode ) {
      	 $this->error_message = '';  //$msg_mode = lang('login_managemode');
      }
      return $msg;
   }

   //Gets the parity and actual suveryid ($usr_surveyid in hex format)
   //First 2 characters is the parity and then the actual survey id
   private function _get_real_surveyid($usr_surveyid) {
		$this->survey_parity = 0;
		$this->survey_id = 0;
		$ret = osa_hex_id_numval($usr_surveyid);
		if ( !$ret ) {
			return FALSE;
		}
		$this->survey_parity = $ret['parity'];
		$this->survey_id = $ret['id'];
		return TRUE;
   }

   private function _is_valid_survey() {
		$survey_record = FALSE;
		$survey_code = $this->survey_code;
		if ( empty($survey_code) ) {
			return FALSE;
		}
		if ( $this->_get_real_surveyid($survey_code) ) {
			$survey_record = $this->survey_model->get_survey($this->survey_id, $this->survey_parity);
		}
		$this->load->model('goal/goal_model');
		if ( !$survey_record ) {
			$this->error_message = lang('survey_login_invalid_code');
			return FALSE;
		}
		// Only enabled goal can do survey so the other goal statuses provide expired message
		//else if ( $this->survey_model->survey_expired($survey_record) ||
		//			( $this->goal_model->has_goal_by_surveyid($this->survey_id) && !$this->goal_model->is_goal_enabled_by_surveyid($this->survey_id) ) ) {
		else if ( $this->survey_model->is_survey_expired_or_goal_readonly($survey_record) ){
			$this->survey_expired = TRUE;
			$this->error_message = lang('survey_expired_msg');
			return FALSE;
		}

		if ( $this->_get_user() ) {
			$user = & $this->user;
			if ( $completed = $user->get_completed($this->survey_code) ) {
				//This will cause the completed survey to come up.
				//$user->resultid = $completed['resultid'];
				//$this->_add_survey2login(FALSE);
				return TRUE;
			}
			if ( !$user->is_available() && $user->get_surveyid() != $this->survey_id ) {
				$survey_obj = $user->get_surveyrecord();
				$link = $this->_survey_login_link($user->get_surveycode());
				$this->error_message = sprintf(lang('survey_disallow_multiple_surveys'), $survey_obj->title,
					"href='{$link}'");
				return FALSE;
			}
		}

		return $survey_record;
	}

	private function _show_survey_link($survey_code='') {
		//return base_url() . self::LINK_SHOW_SURVEY . $this->survey_code;
		return base_url() . self::LINK_SHOW_SURVEY . $survey_code;
	}

	private function _survey_login_link($survey_code=FALSE) {
		if ( !$survey_code ) {
			$survey_code = $this->survey_code;
		}
		return base_url() . self::LINK_SURVEY_LOGIN . $survey_code;
	}

	//Returns true is survey does not require password and security code
	private function _is_open_survey() {
		$survey_obj = $this->survey_record;
		if ( is_object($survey_obj) && !$survey_obj->islogin && empty($survey_obj->code) ) {
			return TRUE;
		}
		return FALSE;
	}

	//Returns true if survey requires security code
	private function _require_security_code() {
		$survey_obj = $this->survey_record;
		if ( is_object($survey_obj) && !$survey_obj->islogin && !empty($survey_obj->code) ) {
			return TRUE;
		}
		return FALSE;
	}

	private function _add_survey2login($check_available=TRUE) {
		if ( ! $this->_get_user() ) {
			return FALSE;
		}
		$user = & $this->user;
   	$user->set_survey($this->survey_code, $this->survey_id, $this->survey_parity, $this->survey_record, $check_available);
   	$this->_set_user();
      return TRUE;
	}

	private function _get_user() {
		if ( !$this->user ) {
			$this->user = $this->survey_login_model->getLogin();
		}
		return $this->user;
	}

	private function _set_user() {
		$this->survey_login_model->setLogin($this->user);
	}
}
