<?php
/**
 * lt.php - Learning Targets controll file
 *
 * @author $Author: dtong $
 * @version $Id: lt.php,v 1.31 2013/09/10 03:38:21 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/

class lt extends CI_Controller {

	function __construct() {
		parent::__construct();
		//$this->load->helper('form');
		$this->load->model('unit/unit_model');
		$this->load->model('unit/lt_model');
		osa_load_lang('unit');
		//if ( !$this->access_model->checkaccess_any('unit_read', FALSE, FALSE, FALSE) ) {
		if ( !osa_check_access(array('unit_write')) ) {
			//A view won't print to the client until this returns and an exist
			//would result in no output. So, just echo it here. Noticed the 3rd parameter in the load->view
			echo $this->load->view('unit/unit_htmlerror_view', array('errormsg'=>lang('unit_no_permissionmsg')),TRUE);
			exit;
		}
	}

	function toggle($unitid, $type) {
		if ( $lib = osa_3p_load_lib('CMS_3P_LIB_BENCH_LT_HIDE', 'CMS_3P_FOLDER_BENCH_LT') ) {
			$lib->toggle();
		}
		$this->add($unitid, $type);
	}

	// Show the LT popup page where an editor can select learning targets.
	function add($unitid, $type) {
		if ( !osa_is_int_one($unitid) || !osa_is_int_one($type) ) {
			$this->load->view('template/ajax_error_view',
				array('errormsg' => lang('unit_no_permissionmsg'), 'divid' => 'learningtarget'));
			return;
		}

		$this->load->library('cms/cms_template/cms_template');
		$template_id = $this->cms_template->type_templateid($type);

		$unit_courseid = 0;
		if ( $template_id == CMS_TEMPLATE_UNIT_IDENTIFIER ) {
			//Only editors can run this, exits if false
			if ( ($unit = $this->unit_model->check_unit_editaccess($unitid)) === FALSE) {
				$this->load->view('template/ajax_error_view',
					array('errormsg' => lang('gen_internalservererror'), 'divid' => 'learningtarget'));
				return;
			}
			$unit_courseid = $unit->courseid;
			$this->load->library('unit/unitmain',
				array('action'=>CMS_ACTION_ADD,'unitid'=>$unitid, 'courseid'=>$unit_courseid));
			$template_refresh_url_part = "unit/unit/stage/";
		}
		elseif ( $template_id == CMS_TEMPLATE_GOAL_IDENTIFIER ) {
			// Unitid is actually goalid.
			if ( !$this->_can_edit_goal($unitid) ) {
				$this->load->view('template/ajax_error_view',
				array('errormsg' => lang('gen_internalservererror'), 'divid' => 'learningtarget'));
				return;
			}
			// Right now, this can only be the goal type besides the unit type.
			// Tell the unitmain object to load the goal template.
			$this->load->library('unit/unitmain',
				array('action'=>CMS_ACTION_ADD,'unitid'=>$unitid, 'courseid'=>$unit_courseid, 'unit_template'=> CMS_GOAL_TEMPLATE));

			$template_refresh_url_part = "goal/goal/stage/";
		}
		else {
			// Template type is not unit or goal and we don't know how to handle this.
			// So, just stop procesing and return error.
			osa_log(__METHOD__, "Don't know how to handle template type " . $template_id);
			$this->load->view('template/ajax_error_view',
				array('errormsg' => lang('gen_internalservererror'), 'divid' => 'learningtarget'));
			return;
		}

		//Get the lt template object
		$ltobj = $this->unitmain->get_lt_obj($type, $unitid);

		if ( !$ltobj || $ltobj->template_data() == FALSE ) {
			osa_errorlog(__METHOD__ . " - Failed to get template type:$type unit:$unitid.");
			$this->load->view('template/ajax_error_view',
				array('errormsg' => lang('gen_internalservererror'), 'divid' => 'learningtarget'));
			return;
		}

		// $cms_lt_importid_support = $this->config->item('cms_lt_importid_support');
		// cms_lt_importid_support was desgined to be a global param but we needed to make it module based.
		// Therefore we are resetting the value here. No need to set it back to the orig value.
		$cms_lt_importid_support = $ltobj->getval('_display_importid');
		$this->config->set_item('cms_lt_importid_support', $cms_lt_importid_support);

		if ( $ltobj->getval('_bench_lt_hide_disable') === TRUE ) {
			// Turns off the 3P hide used LT feature.
			$this->config->set_item('cms_3p_bench_lt_hide_disable', TRUE);
		}
		if ( $ltobj->getval('_use_all_departments') === TRUE ) {
			// Query used for this flag also ignores grade levels; gets all benchmarks regardless of grade levels.
			// Sql3 in 3p bench_lt_model.php.
			$this->config->set_item('cms_3p_bench_lt_use_all_departments', TRUE);
		}

		//$sort_type = osa_template_sort_type($ltobj->template_data());
		//Get the sort type from the lt edit template which different than the non-edit template
		$sort_type = osa_template_sort_type($this->unitmain->get_lt_edittemplate());
		$data = new stdClass();
		$data->config_hide_benchmarks = $ltobj->getval('_hide_benchmarks');
		$data->template_refresh_url_part = $template_refresh_url_part;
		$data->divid = $ltobj->getval('_divid');
		if ( empty($data->divid) ) {
			$data->divid = 'learningtarget';
		}
		if ( $lib = osa_3p_load_lib('CMS_3P_LIB_BENCH_LT_HIDE', 'CMS_3P_FOLDER_BENCH_LT') ) {
			$lib->html($data, $unitid, $type);
		}
		if ( $ltobj->getval('_hide_benchmarks') === TRUE ) {
			$this->config->set_item('cms_hide_benchmarks', TRUE);
		}
		$data->sort_type = $sort_type;
		$data->courseid = $unit_courseid;
		$this->lt_model->sort_type = $sort_type;
		$this->lt_model->sort_filehash = $this->unitmain->get_template_filepart();
		$ltdbobj = $this->lt_model->get_selections($unit_courseid, $unitid, $type);
		// $this->lt_model->linked_courses is set inside lt_model->get_selections.
		// TODO: move the logic of this query into the big query above.
		$depts = $this->unit_model->get_department_display($this->lt_model->linked_courses, 'courseid', TRUE);

		$data->db_flag = $ltdbobj ? TRUE : FALSE;
		$data->template = $ltobj;
		$last_standard = FALSE;
		$last_bench = FALSE;
		$db_struct = array();
		$depts_name = array();
		$last_deptid = FALSE;
		$selectedids = FALSE;
		$anchors = array();
		$last_strandid = FALSE;

		if ( $ltdbobj ) {
			$array_keys = array_keys($ltdbobj);
			//foreach($ltdbobj as $record) {
			foreach ($array_keys as $key) {
				$record = &$ltdbobj[$key];
				if ( $last_standard != $record->standardid) {
					$db_struct[$record->standardid] = array();
					if ( $cms_lt_importid_support ) {
						$db_struct[$record->standardid][] = "$record->standard_importid $record->standard";
						if ( $last_strandid != $record->strandid ) {
							$db_struct[$record->standardid]['strand'] = &$record->strand;
							$db_struct[$record->standardid]['strand_importid'] = &$record->strand_importid;
							$last_strandid = &$record->strandid;
						}
					}
					else {
						$db_struct[$record->standardid][] = &$ltdbobj[$key]->standard;
						if ( $last_strandid != $record->strandid ) {
							$db_struct[$record->standardid]['strand'] = &$record->strand;
							$last_strandid = &$record->strandid;
						}
					}
					$last_standard = &$ltdbobj[$key]->standardid;
				}
				if ($last_bench != $record->benchmarkid) {
					$db_struct[$record->standardid][$record->benchmarkid] = array();
					if ( $cms_lt_importid_support ) {
						$db_struct[$record->standardid][$record->benchmarkid][] = "$record->benchmark_importid $record->benchmark";
					}
					else {
						$db_struct[$record->standardid][$record->benchmarkid][] = &$ltdbobj[$key]->benchmark;
					}
					$last_bench = &$ltdbobj[$key]->benchmarkid;
				}
				$db_struct[$record->standardid][$record->benchmarkid][] = &$ltdbobj[$key];
				if ( $last_deptid === FALSE || $last_deptid != $record->deptid ) {
					if ( isset($record->courseid) && isset($depts[$record->courseid]) ) {
						// Think this is for department_display override set in a course record.
						// Doesn't work when using benchmarks as LTs.
						$deptname = &$depts[$record->courseid]->name;
					}
					else {
						$deptname = &$ltdbobj[$key]->deptname;
					}
					$record->deptname = $deptname;
					$depts_name[$record->standardid] = $record;
					$last_deptid = $record->deptid;
					$anchors[$record->standardid] = array("anchor_{$record->standardid}_$unitid", $record->deptname);
				}
			}
		}
		$data->db_struct = &$db_struct;
		$data->depts_name = $depts_name;
		$data->courseid = $unit_courseid;
		if ( empty($ltdbobj) ) {
			// Counting boolean false returns 1 so have to test empty.
			$data->lt_count = 0;
		}
		else {
			$data->lt_count = count($ltdbobj);
		}
		if ( count($anchors) > 1 && $data->lt_count > 10 ) {
			$data->anchors = &$anchors;
		}
		else {
			$data->anchors = array();
		}

		if ( $data->template->getval('_hide_strand_title') === TRUE ) {
			$data->strand_title = '';
		}
		else {
			$data->strand_title = $ltobj->getval('_strand_title');
			if ( empty($data->strand_title) ) {
				$data->strand_title = lang('unit_lt_popup_strand');
			}
			else {
				$data->strand_title .= ': ';
			}
		}
		if ( $ltobj->getval('_big_std_separator') ) {
			$data->std_separator = 'ltwin_sep3';
		}
		else {
			$data->std_separator = 'ltwin_sep1';
		}
		$data->deparetment_name_override = $ltobj->getval('_department_name_override');
		$this->load->view('unit/unit_ltadd_view', $data);
	}

	function update($unitid, $type) {
		if ( !osa_is_int($unitid) || !osa_is_int($type) ) {
			$this->load->view('template/ajax_error_view',
				array('errormsg' => lang('unit_no_permissionmsg'), 'divid' => 'learningtarget'));
			return;
		}
		/*
		//Only editors can run this, exits if false
		if ( ($unit = $this->unit_model->check_unit_editaccess($unitid)) === FALSE) {
			$this->load->view('template/ajax_error_view',
				array('errormsg' => lang('gen_internalservererror'), 'divid' => 'learningtarget'));
			return;
		}
		*/
		$this->load->library('cms/cms_template/cms_template');
		$template_id = $this->cms_template->type_templateid($type);

		if ( $template_id == CMS_TEMPLATE_UNIT_IDENTIFIER ) {
			//Only editors can run this, exits if false
			if ( ($unit = $this->unit_model->check_unit_editaccess($unitid)) === FALSE) {
				$this->load->view('template/ajax_error_view',
				array('errormsg' => lang('gen_internalservererror'), 'divid' => 'learningtarget'));
				return;
			}
		}
		elseif ( $template_id == CMS_TEMPLATE_GOAL_IDENTIFIER ) {
			// Unitid is actually goalid.
			if ( !$this->_can_edit_goal($unitid) ) {
				$this->load->view('template/ajax_error_view',
					array('errormsg' => lang('gen_internalservererror'), 'divid' => 'learningtarget'));
				return;
			}
		}
		else {
			// Template type is not unit or goal and we don't know how to handle this.
			// So, just stop procesing and return error.
			osa_log(__METHOD__, "Don't know how to handle template type " . $template_id);
			$this->load->view('template/ajax_error_view',
				array('errormsg' => lang('gen_internalservererror'), 'divid' => 'learningtarget'));
			return;
		}

		$keys = array_keys($_POST);
		$pattern = 'lt';
		$ids = array();
		foreach($keys as $key) {
			$tok = strtok($key, '_');
			if ($tok==$pattern) {
				$formid = strtok('_');
				if ( is_numeric($formid)) {
					if ( is_numeric($_POST[$key]) ) {
						$ids[$formid] = $_POST[$key];
					}
				}
			}
		}

		$no_delete=FALSE;
		if ( isset($_POST['no_delete']) && $_POST['no_delete'] ) {
			$no_delete = TRUE;
		}
		if ( ($this->lt_model->update($unitid, $type, $ids, $no_delete)) === FALSE ) {
			echo osa_ajaxmsg(lang('unit_failedtoadddata'));
			exit;
		}
		osa_exit(CMS_AJAX_SUCCESS);
	}

	function delete ($unitid, $targetid) {
		$this->unit_model->check_unit_editaccess($unitid);

		if ( !osa_is_int($targetid) ) {
			if ( $lib = osa_3p_load_lib('CMS_3P_LIB_BENCH_LT', 'CMS_3P_FOLDER_BENCH_LT') ) {
				$lib->delete($unitid, $targetid);
			}
			exit;
		}

		if ( !$this->lt_model->delete($unitid, $targetid) ) {
			echo osa_ajaxmsg(lang('unit_failedtodeletedata'));
      	exit;
		}
	}

	// Check to see if current user can edit the goal pointed to by the supplied goal id.
	private function _can_edit_goal($goalid) {
		if ( !osa_is_int_one($goalid) ) {
			return FALSE;
		}
		$this->load->model('goal/goal_model');
		return $this->goal_model->is_edit_goal($goalid);
	}
}

